import unittest

import pyminknow.config

import minknow_api.manager


class TestProtocolService(unittest.TestCase):
    """Test protocol service"""

    def setUp(self) -> None:
        """Initialise server"""

        # Don't use SSL certificates
        self.client = minknow_api.manager.Manager(use_tls=False)

    def test_describe_host(self):
        host = self.client.describe_host()

        self.assertEqual(host.product_code, pyminknow.config.PRODUCT_CODE)
        self.assertEqual(host.description, pyminknow.config.DESCRIPTION)
        self.assertEqual(host.serial, pyminknow.config.SERIAL)
        self.assertEqual(host.network_name, pyminknow.config.NETWORK_NAME)

    def test_flow_cell_positions(self):
        devices = {d['name']: d for d in pyminknow.config.DEVICES}

        for flow_cell_position in self.client.flow_cell_positions():
            d = devices[flow_cell_position.name]

            self.assertEqual(flow_cell_position.description.rpc_ports.insecure, d['ports']['insecure'])



import logging
import warnings

import minknow_api.manager_pb2
import minknow_api.manager_pb2_grpc

import pyminknow.config

LOGGER = logging.getLogger(__name__)


class ManagerService(minknow_api.manager_pb2_grpc.ManagerServiceServicer):
    """
    Manager service

    https://github.com/nanoporetech/minknow_lims_interface/blob/master/minknow/rpc/manager.proto
    """
    add_to_server = minknow_api.manager_pb2_grpc.add_ManagerServiceServicer_to_server

    def get_version_info(self, request, context):
        return minknow_api.manager_pb2.GetVersionInfoResponse()

    def describe_host(self, request, context):
        return minknow_api.manager_pb2.DescribeHostResponse(
            product_code=pyminknow.config.PRODUCT_CODE,
            description=pyminknow.config.DESCRIPTION,
            serial=pyminknow.config.SERIAL,
            network_name=pyminknow.config.NETWORK_NAME,
        )

    @property
    def active_devices(self) -> list:
        return [
            minknow_api.manager_pb2.ListDevicesResponse.ActiveDevice(
                name=device['name'],
                layout=minknow_api.manager_pb2.ListDevicesResponse.DeviceLayout(**device['layout']),
                ports=minknow_api.manager_pb2.ListDevicesResponse.RpcPorts(
                    insecure_grpc=device['ports']['insecure'],
                    secure=device['ports']['secure'],
                )
            )
            for device in pyminknow.config.DEVICES
        ]

    def list_devices(self, request, context):
        warnings.warn('Use `flow_cell_positions` instead', DeprecationWarning)

        return minknow_api.manager_pb2.ListDevicesResponse(
            active=self.active_devices,
            inactive=list(),
            pending=list(),
        )

    def flow_cell_positions(self, request, context) -> iter:
        """Provides a snapshot of places where users can insert flow cells."""

        positions = self._flow_cell_positions

        yield minknow_api.manager_pb2.FlowCellPositionsResponse(
            total_count=len(positions),
            positions=positions,
        )

    @property
    def _flow_cell_positions(self) -> list:
        state_name = 'STATE_RUNNING'
        state = minknow_api.manager_pb2.FlowCellPosition.State.Value(state_name)

        return [
            minknow_api.manager_pb2.FlowCellPosition(
                name=device['name'],
                location=minknow_api.manager_pb2.FlowCellPosition.Location(**device['layout']),
                state=state,
                rpc_ports=minknow_api.manager_pb2.FlowCellPosition.RpcPorts(**device['ports']),
                error_info='',
            ) for device in pyminknow.config.DEVICES
        ]



import logging
import unittest

import google.protobuf.internal.containers
import minknow_api.tools.protocols
import minknow_api.protocol_pb2

import pyminknow.config

# Use the first device
PORT = pyminknow.config.DEVICES[0]['ports']['insecure']


class TestProtocolService(unittest.TestCase):
    """Test protocol service"""

    def setUp(self) -> None:
        """Initialise server"""

        logging.basicConfig(level=logging.WARNING)

    @unittest.skip
    def test_start_protocol(self):
        # TODO
        pass
        # minknow_api.tools.protocols.start_protocol()
        # response = self.client.start_protocol(
        #     identifier='sequencing/sequencing_MIN106_DNA:FLO-MIN106:SQK-LSK109:True',
        #     user_info=dict(
        #         protocol_group_id='covid19-20200512-1589297081',
        #         sample_id='covid19-20200512-1589297081',
        #     ),
        #     args=[
        #         "--fast5=on",
        #         "--fast5_data", "trace_table", "fastq", "raw", "zlib_compress",
        #         "--base_calling=on",
        #         "--fastq=on",
        #         "--barcoding_kits", "EXP-NBD114", "EXP-NBD104",
        #         "--experiment_time=24"
        #     ],
        # )
        #
        # self.assertIsInstance(response, minknow_api.protocol_pb2.StartProtocolResponse)
        # run_id = response.run_id
        #
        # self.assertIsInstance(run_id, str)
        #
        # return run_id

    @unittest.skip
    def test_get_run_info(self):
        run_id = self.test_start_protocol()

        protocol_run_info = self.client.get_run_info(run_id)

        self.assertIsInstance(protocol_run_info, minknow_api.protocol_pb2.ProtocolRunInfo)

        self.assertEqual(run_id, protocol_run_info.run_id)

    @unittest.skip
    def test_list_protocols(self):
        response = self.client.list_protocols()
        self.assertIsInstance(response, minknow_api.protocol_pb2.ListProtocolsResponse)

        for protocol in response.protocols:
            self.assertIsNotNone(protocol.identifier)
            self.assertIsInstance(protocol.identifier, str)

    @unittest.skip
    def test_list_protocol_runs(self):
        response = self.client.list_protocol_runs()
        self.assertIsInstance(response, minknow_api.protocol_pb2.ListProtocolRunsResponse)
        self.assertIsInstance(response.run_ids, google.protobuf.internal.containers.RepeatedScalarFieldContainer)

        for run_id in response.run_ids:
            self.assertIsInstance(run_id, str)


if __name__ == '__main__':
    unittest.main()
